@@ -3,6 +3,8 @@ package App::ManiacDownloader;
 use strict;
 use warnings;
 
+use autodie;
+
 use MooX qw/late/;
 use URI;
 use AnyEvent::HTTP qw/http_head http_get/;
@@ -10,10 +12,11 @@ use Getopt::Long qw/GetOptionsFromArray/;
 use File::Basename qw(basename);
 use Fcntl qw( SEEK_SET );
 use List::UtilsBy qw(max_by);
+use JSON qw(decode_json encode_json);
 
 use App::ManiacDownloader::_SegmentTask;
 
-our $VERSION = '0.0.2';
+our $VERSION = '0.0.3';
 
 my $DEFAULT_NUM_CONNECTIONS = 4;
 my $NUM_CONN_BYTES_THRESHOLD = 4_096 * 2;
@@ -29,6 +32,19 @@ has '_stats_timer' => (is => 'rw');
 has '_last_timer_time' => (is => 'rw', isa => 'Num');
 has '_len' => (is => 'rw', isa => 'Int');
 
+sub _serialize
+{
+    my ($self) = @_;
+
+    return
+    +{
+        _ranges => [map { $_->_serialize() } @{$self->_ranges}],
+        _remaining_connections => $self->_remaining_connections,
+        _bytes_dled => $self->_bytes_dled,
+        _len => $self->_len,
+    };
+}
+
 sub _downloading_path
 {
     my ($self) = @_;
@@ -36,6 +52,13 @@ sub _downloading_path
     return $self->_url_basename . '.mdown-intermediate';
 }
 
+sub _resume_info_path
+{
+    my ($self) = @_;
+
+    return $self->_url_basename . '.mdown-resume.json';
+}
+
 sub _start_connection
 {
     my ($self, $idx) = @_;
@@ -109,6 +132,99 @@ sub _handle_stats_timer
     return;
 }
 
+sub _slurp
+{
+    my $filename = shift;
+
+    open my $in, '<', $filename
+        or die "Cannot open '$filename' for slurping - $!";
+
+    local $/;
+    my $contents = <$in>;
+
+    close($in);
+
+    return $contents;
+}
+
+sub _init_from_len
+{
+    my ($self, $args) = @_;
+
+    my $num_connections = $args->{num_connections};
+    my $len = $self->_len;
+    my $url_basename = $self->_url_basename;
+
+    my @stops = (map { int( ($len * $_) / $num_connections ) }
+        0 .. ($num_connections-1));
+
+    push @stops, $len;
+
+    my @ranges = (
+        map {
+        App::ManiacDownloader::_SegmentTask->new(
+        _start => $stops[$_],
+        _end => $stops[$_+1],
+        )
+        }
+        0 .. ($num_connections-1)
+    );
+
+    $self->_ranges(\@ranges);
+
+    my $ranges_ref = $args->{ranges};
+    foreach my $idx (0 .. $num_connections-1)
+    {
+        my $r = $ranges[$idx];
+
+        if (defined($ranges_ref))
+        {
+            $r->_deserialize($ranges_ref->[$idx]);
+        }
+
+        if ($r->is_active)
+        {
+            {
+                open my $fh, "+>:raw", $self->_downloading_path()
+                    or die "${url_basename}: $!";
+
+                $r->_fh($fh);
+            }
+
+            $self->_start_connection($idx);
+        }
+    }
+
+    my $timer = AnyEvent->timer(
+        after => 3,
+        interval => 3,
+        cb => sub {
+            $self->_handle_stats_timer;
+            return;
+        },
+    );
+    $self->_last_timer_time(AnyEvent->time());
+    $self->_stats_timer($timer);
+
+    {
+        no autodie;
+        unlink($self->_resume_info_path());
+    }
+
+    return;
+}
+
+sub _abort_signal_handler
+{
+    my ($self) = @_;
+
+    open my $json_out_fh, '>:encoding(utf8)', $self->_resume_info_path();
+    print {$json_out_fh} encode_json($self->_serialize);
+    close ($json_out_fh);
+
+    exit(2);
+}
+
 sub run
 {
     my ($self, $args) = @_;
@@ -136,65 +252,55 @@ sub run
 
     $self->_url_basename($url_basename);
 
+    if (-e $self->_url_basename)
+    {
+        print STDERR "File appears to have already been downloaded. Quitting.\n";
+        return;
+    }
+
     $self->_finished_condvar(
         scalar(AnyEvent->condvar)
     );
 
-    http_head $url, sub {
-        my (undef, $headers) = @_;
-        my $len = $headers->{'content-length'};
-
-        if (!defined($len)) {
-            die "Cannot find a content-length header.";
-        }
-
-        $self->_len($len);
-
-        my @stops = (map { int( ($len * $_) / $num_connections ) }
-            0 .. ($num_connections-1));
-
-        push @stops, $len;
-
-        my @ranges = (
-            map {
-                App::ManiacDownloader::_SegmentTask->new(
-                    _start => $stops[$_],
-                    _end => $stops[$_+1],
-                )
-            }
-            0 .. ($num_connections-1)
-        );
-
-        $self->_ranges(\@ranges);
-
-        $self->_remaining_connections($num_connections);
-        foreach my $idx (0 .. $num_connections-1)
-        {
-            my $r = $ranges[$idx];
-
+    if (-e $self->_resume_info_path)
+    {
+        my $record = decode_json(_slurp($self->_resume_info_path));
+        $self->_len($record->{_len});
+        $self->_bytes_dled($record->{_bytes_dled});
+        $self->_bytes_dled_last_timer($self->_bytes_dled());
+        $self->_remaining_connections($record->{_remaining_connections});
+        my $ranges_ref = $record->{_ranges};
+        $self->_init_from_len(
             {
-                open my $fh, "+>:raw", $self->_downloading_path()
-                    or die "${url_basename}: $!";
+                ranges => $ranges_ref,
+                num_connections => scalar(@$ranges_ref),
+            }
+        );
+    }
+    else
+    {
+        http_head $url, sub {
+            my (undef, $headers) = @_;
+            my $len = $headers->{'content-length'};
 
-                $r->_fh($fh);
+            if (!defined($len)) {
+                die "Cannot find a content-length header.";
             }
 
-            $self->_start_connection($idx);
-        }
+            $self->_len($len);
+            $self->_remaining_connections($num_connections);
 
-        my $timer = AnyEvent->timer(
-            after => 3,
-            interval => 3,
-            cb => sub {
-                $self->_handle_stats_timer;
-                return;
-            },
-        );
-        $self->_last_timer_time(AnyEvent->time());
-        $self->_stats_timer($timer);
+            return $self->_init_from_len(
+                {
+                    num_connections => $num_connections,
+                }
+            );
+        };
+    }
 
-        return;
-    };
+    my $signal_handler = sub { $self->_abort_signal_handler(); };
+    local $SIG{INT} = $signal_handler;
+    local $SIG{TERM} = $signal_handler;
 
     $self->_finished_condvar->recv;
     $self->_stats_timer(undef());
@@ -219,7 +325,7 @@ App::ManiacDownloader - a maniac download accelerator.
 
 =head1 VERSION
 
-version 0.0.2
+version 0.0.3
 
 =head1 SYNOPSIS
 
